<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
	<xsl:param name="dbschema">TCTEST</xsl:param>
	<xsl:param name="package">de.cegedim.iss.tc.bdo</xsl:param>
	<xsl:template match="/">
		<descriptor-repository>
			<jdbc-connection-descriptor/>
			<xsl:apply-templates select="app-data/database/table"/>
		</descriptor-repository>
	</xsl:template>
	<xsl:template match="table">
		<class-descriptor>
			<xsl:attribute name="class"><xsl:variable name="class-name"><xsl:call-template name="class-name-for-table"><xsl:with-param name="table-name" select="@name"/></xsl:call-template></xsl:variable><xsl:value-of select="concat($package,'.',$class-name)"/></xsl:attribute>
			<xsl:attribute name="table"><xsl:value-of select="concat($dbschema,'.',@name)"/></xsl:attribute>
			<description>TODO: review descriptor generated from table structure</description>
			<xsl:apply-templates select="column"/>
		</class-descriptor>
	</xsl:template>
	<xsl:template match="table">
		<class-descriptor>
			<xsl:attribute name="class"><xsl:variable name="class-name"><xsl:call-template name="jdbc-to-java-name"><xsl:with-param name="jdbc-name" select="@name"/></xsl:call-template></xsl:variable><xsl:value-of select="concat($package,'.',$class-name)"/></xsl:attribute>
			<xsl:attribute name="table"><xsl:value-of select="concat($dbschema,'.',@name)"/></xsl:attribute>
			<xsl:apply-templates select="column"/>
		</class-descriptor>
	</xsl:template>
	<xsl:template match="column">
		<field-descriptor>
			<xsl:attribute name="id"><xsl:value-of select="position()"/></xsl:attribute>
			<xsl:attribute name="name"><xsl:call-template name="jdbc-to-java-name-field"><xsl:with-param name="jdbc-name" select="@name"/></xsl:call-template></xsl:attribute>
			<xsl:attribute name="column"><xsl:value-of select="@name"/></xsl:attribute>
			<!--apply some heuristics about DECIMAL-->
			<xsl:choose>
				<xsl:when test="@type='DECIMAL' and @name='UPDATE_COUNT'">
					<xsl:attribute name="jdbc-type">INTEGER</xsl:attribute>
				</xsl:when>
				<xsl:when test="@type='DECIMAL' and @primaryKey">
					<xsl:attribute name="jdbc-type">BIGINT</xsl:attribute>
				</xsl:when>
				<xsl:when test="@type='DECIMAL' and substring(@name,string-length(@name)-1)='ID'">
					<xsl:attribute name="jdbc-type">BIGINT</xsl:attribute>
				</xsl:when>
				<xsl:when test="@type='TIMESTAMP'">
					<xsl:attribute name="jdbc-type">TIMESTAMP</xsl:attribute>
					<xsl:attribute name="conversion">org.apache.ojb.broker.accesslayer.conversions.JavaDate2SqlTimestampFieldConversion</xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="jdbc-type"><xsl:value-of select="@type"/></xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:if test="@size">
				<xsl:attribute name="length"><xsl:value-of select="@size"/></xsl:attribute>
			</xsl:if>
			<xsl:if test="@required">
				<xsl:attribute name="nullable"><xsl:value-of select="not(@required='true')"/></xsl:attribute>
			</xsl:if>
			<xsl:if test="@primaryKey">
				<xsl:attribute name="primarykey"><xsl:value-of select="@primaryKey"/></xsl:attribute>
				<xsl:attribute name="autoincrement">true</xsl:attribute>
			</xsl:if>
			<xsl:if test="@name='UPDATE_COUNT'">
				<xsl:attribute name="locking">true</xsl:attribute>
			</xsl:if>
		</field-descriptor>
	</xsl:template>
	<!--JDBC name (all capital letters) to java name (with first capital letter, e.g. a class name)-->
	<xsl:template name="jdbc-to-java-name">
		<xsl:param name="jdbc-name"/>
		<xsl:choose>
			<xsl:when test="contains($jdbc-name,'_')">
				<xsl:variable name="beforeUnderscore">
					<xsl:call-template name="jdbc-to-java-name">
						<xsl:with-param name="jdbc-name">
							<xsl:value-of select="substring-before($jdbc-name,'_')"/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:variable>
				<xsl:variable name="afterUnderscore">
					<xsl:call-template name="jdbc-to-java-name">
						<xsl:with-param name="jdbc-name">
							<xsl:value-of select="substring-after($jdbc-name,'_')"/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:variable>
				<xsl:value-of select="concat($beforeUnderscore,$afterUnderscore)"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="name-lower">
					<xsl:value-of select="translate($jdbc-name, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz' )"/>
				</xsl:variable>
				<xsl:value-of select="concat(substring($jdbc-name,1,1),substring($name-lower,2))"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--JDBC name (all capital letters) to java name (with first small letter, e.g. a field name)-->
	<xsl:template name="jdbc-to-java-name-field">
		<xsl:param name="jdbc-name"/>
		<xsl:variable name="java-name">
			<xsl:call-template name="jdbc-to-java-name">
				<xsl:with-param name="jdbc-name">
					<xsl:value-of select="$jdbc-name"/>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="name-lower">
			<xsl:value-of select="translate($jdbc-name, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz' )"/>
		</xsl:variable>
		<xsl:value-of select="concat(substring($name-lower,1,1),substring($java-name,2))"/>
	</xsl:template>
</xsl:stylesheet>

